<?php

namespace Litepie\Role\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Litepie\Role\Events\RoleAction as RoleActionEvent;
use Litepie\Role\Notifications\RoleAction as RoleActionNotification;
use Litepie\Actions\Concerns\AsAction;

class RoleAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the RoleActionEvent.
     *
     * @param   RoleActionEvent  $event
     * @return mixed
     */
    public function handle(RoleActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $role.
     *
     * @param   RoleActionEvent  $event
     * @return void
     */
    public function create(RoleActionEvent $event)
    {
        $client = $event->role->client;
        Notification::send($client, new RoleActionNotification($event));
    }

    /**
     * Handle the RoleActionEvent as a listener.
     *
     * @param   RoleActionEvent  $event
     * @return mixed
     */
    public function asListener(RoleActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   RoleActionEvent  $event
     * @return bool
     */
    private function isAllowed(RoleActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
